local function handle_ranks(...)
	local result = {}
	for i = 1, select("#", ...) do
		result[i] = tonumber((select(i, ...)))
	end
	return result
end

local function handle_talents(...)
	local result = {}
	for talent = 1, select("#", ...) do
		result[talent] = handle_ranks(strsplit(";", (select(talent, ...))))
	end
	return result
end

local function handle_tabs(...)
	local result = {}
	for tab = 1, select("#", ...) do
		result[tab] = handle_talents(strsplit(",", (select(tab, ...))))
	end
	return result
end

function Talented:UncompressSpellData(class)
	local data = self.spelldata[class]
	if type(data) == "table" then return data end
	data = handle_tabs(strsplit("|", data))
	self.spelldata[class] = data
	return data
end

local spellTooltip
local function CreateSpellTooltip()
	local tt = CreateFrame"GameTooltip"
	local lefts = {}
	for i = 1, 20 do
		local left, right = tt:CreateFontString(), tt:CreateFontString()
		left:SetFontObject(GameFontNormal)
		right:SetFontObject(GameFontNormal)
		tt:AddFontStrings(left, right)
		lefts[i] = left
	end
	tt.lefts = lefts
	function tt:SetSpell(spell)
		if self.spell ~= spell then
			self:SetOwner(TalentedFrame)
			self:ClearLines()
			self:SetHyperlink("spell:"..spell)
			self.spell = spell
		end
	end
	Talented.spellDescCache = setmetatable({}, {
		__index = function (self, key)
			tt:SetSpell(key)
			local value = tt.lefts[tt:NumLines()]:GetText()
			self[key] = value
			return value
		end,
	})
	CreateSpellTooltip = nil
	return tt
end

function Talented:GetTalentName(class, tab, index)
	local spell = self:UncompressSpellData(class)[tab][index][1]
	return (GetSpellInfo(spell))
end

function Talented:GetTalentIcon(class, tab, index)
	local spell = self:UncompressSpellData(class)[tab][index][1]
	return (select(3, GetSpellInfo(spell)))
end

function Talented:GetTalentDesc(class, tab, index, rank)
	if not spellTooltip then
		spellTooltip = CreateSpellTooltip()
	end
	local spell = self:UncompressSpellData(class)[tab][index][rank]
	return self.spellDescCache[spell]
end

function Talented:GetTalentLink(template, tab, index, rank)
	local data = self:UncompressSpellData(template.class)
	local rank = rank or (template[tab] and template[tab][index])
	if not rank or rank == 0 then
		rank = 1
	end
	return
		("|cff71d5ff|Hspell:%d|h[%s]|h|r"):format(data[tab][index][rank], self:GetTalentName(template.class, tab, index))
end
